# -*- coding: utf-8 -*-
"""
Created on Tue Aug  4 08:19:31 2020

@author: timhe
"""
import os
import requests
import datetime
import pandas as pd

parameters = ["Psurf_f_inst"]
latlim = [25.313, 59.063]
lonlim = [-121.641, -82.969]
Startdate = "2001-01-01"
Enddate = "2001-01-03"
Version = "2.1"
Model = "GLDAS_NOAH025_M"
output_folder = r"F:\PHD\PHD_2\Datasets_Mead_Farm"

"GLDAS_NOAH025_M"  2.1 en 2.0
"GLDAS_NOAH025_3H" 2.1 en 2.0
"GLDAS_NOAH10_3H"  2.1 en 2.0
"GLDAS_NOAH10_M"   2.1 en 2.0

"GLDAS_CLSM025_D"  2.0
"GLDAS_CLSM10_3H"  2.1
"GLDAS_CLSM10_M"   2.1

"GLDAS_VIC10_M"    2.1
"GLDAS_VIC10_3H"   2.1








dates = pd.date_range(Startdate, Enddate, freq = "3H")
filename_temp_nc_format = "temp_{model}_{version}_{year}{month:02d}{day:02d}_{hour:02d}{minute:02d}_{parameter_list}.nc"

for date in dates:
    
    year = date.year
    month = date.month
    day = date.day
    doy = date.dayofyear
    hour = date.hour
    minute = date.minute
    version = Version.replace(".","")
    version_dot = Version
    parameter_list = "-".join(parameters)
    parameter = "%2C".join(parameters)
    url_format = "https://hydro1.gesdisc.eosdis.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fdata%2F{model_first}%2F{model}.{version_dot}%2F{year}%2F{model}.A{year}{month:02d}.0{version}.nc4&FORMAT=bmM0Lw&BBOX={latmin}%2C{lonmin}%2C{latmax}%2C{lonmax}&LABEL={model}.A{year}{month:02d}.0{version}.nc4.SUB.nc4&SHORTNAME={model}&SERVICE=L34RS_LDAS&VERSION=1.02&DATASET_VERSION={version_dot}&VARIABLES={para}"

    url = url_format.format(year = year, month = month, latmin = latlim[0], latmax = latlim[1], lonmin = lonlim[0], lonmax = lonlim[1], para = parameter, version = version, version_dot = version_dot, model = Model, model_first = Model.split("_")[0])
    
    filename_temp_nc = filename_temp_nc_format.format(year = year, month = month, day = day, model = Model, version = version, hour = hour, minute = minute, parameter_list = parameter_list)
    download_file(url, os.path.join(output_folder, filename_temp_nc))




def download_file(dl_url, local_filename):
    
    session = requests.Session()
    downloaded = 0
    with session.get(dl_url, stream=True) as r:
        r.raise_for_status()
        with open(local_filename, 'wb') as f:
            for chunk in r.iter_content(chunk_size=1024 * 1024):
                if chunk:  # filter out keep-alive new chunks
                    f.write(chunk)
                    downloaded += len(chunk)
                    # f.flush()
    return 


class VariablesInfo_NOAH:
   parameters = {'albedo_inst': 'Albedo_inst',
                'avgsurft_inst': 'AvgSurfT_inst',
                'canopint_inst': 'CanopInt_inst',
                'ecanop_tavg': 'ECanop_tavg',
                'esoil_tavg': 'ESoil_tavg',
                'evap_tavg': 'Evap_tavg',
                'lwdown_f_tavg': 'LWdown_f_tavg',
                'lwnet_tavg': 'Lwnet_tavg',
                'potevap_tavg': 'PotEvap_tavg',
                'psurf_f_inst': 'Psurf_f_inst',
                'qair_f_inst': 'Qair_f_inst',
                'qg_tavg': 'Qg_tavg',
                'qh_tavg': 'Qh_tavg',
                'qle_tavg': 'Qle_tavg',
                'qs_acc': 'Qs_acc',
                'qsb_acc': 'Qsb_acc',
                'qsm_acc': 'Qsm_acc',
                'rainf_f_tavg': 'Rainf_f_tavg',
                'rainf_tavg': 'Rainf_tavg',
                'rootmoist_inst': 'RootMoist_inst',
                'snowdepth_inst': 'SnowDepth_inst',
                'snowf_tavg': 'Snowf_tavg',
                'sm0_10cm_ins': 'SoilMoi0_10cm_inst',
                'sm10_40cm_ins': 'SoilMoi10_40cm_inst',
                'sm40_100cm_ins': 'SoilMoi40_100cm_inst',
                'sm100_200cm_ins': 'SoilMoi100_200cm_inst',
                'st0_10cm_ins': 'SoilTMP0_10cm_inst',
                'st10_40cm_ins': 'SoilTMP10_40cm_inst',
                'st40_100cm_ins': 'SoilTMP40_100cm_inst',
                'st100_200cm_ins':'SoilTMP100_200cm_inst',
                'swdown_f_tavg': 'SWdown_f_tavg',
                'swe_inst': 'SWE_inst',
                'swnet_tavg': 'Swnet_tavg',
                'tair_f_inst': 'Tair_f_inst',
                'tveg_tavg' : 'Tveg_tavg',
                'wind_f_inst': 'Wind_f_inst'}
   names = {'albedo_inst': 'Albedo',
             'avgsurft_inst': 'T',
             'canopint_inst': 'TotCanopyWaterStorage',
             'ecanop_tavg': 'ECanop',
             'esoil_tavg': 'ESoil',             
             'evap_tavg': 'ET',
             'lwdown_f_tavg': 'LWdown',
             'lwnet_tavg': 'LWnet',
             'potevap_tavg': 'Epot'             
             'psurf_f_inst': 'P',
             'qair_f_inst': 'Hum',
             'qg_tavg': 'G',
             'qh_tavg': 'H',
             'qle_tavg': 'LE',
             'qs_acc': 'Rsur',
             'qsb_acc': 'Rsubsur',
             'qsm_acc': 'SnowMelt',
             'rainf_f_tavg': 'P',
             'rainf_tavg': 'Prain',
             'rootmoist_inst': 'RootMoist',
             'snowdepth_inst': 'SnowDepth',           
             'swe_inst': 'SnowWaterEquivalent',
             'swdown_f_tavg': 'SWdown',
             'swnet_tavg': 'SWnet',
             'snowf_tavg': 'Snow',
             'sm0_10cm_ins': 'S1',
             'sm10_40cm_ins': 'S2',
             'sm40_100cm_ins': 'S3',
             'sm100_200cm_ins': 'S4',
             'st0_10cm_ins': 'Ts1',
             'st10_40cm_ins': 'Ts2',
             'st40_100cm_ins': 'Ts3',
             'st100_200cm_ins': 'Ts4',
             'tair_f_inst': 'Tair',
             'wind_f_inst': 'W',
             'tveg_tavg' : 'Transpiration'}
    descriptions = {'albedo_inst': 'Albedo [percentage]',
                    'avgsurft_inst': 'surface average skin surface temperature [k]',
                    'canopint_inst': 'surface plant canopy surface water [kg/m^2]',
                    'ecanop_tavg': 'Canopy water evaporation [W/m^2]',
                    'esoil_tavg': 'Direct evaporation from bare soil [W/m^2]',    
                    'evap_tavg': 'surface total evapotranspiration [kg/m^2/s]',
                    'lwdown_f_tavg': 'surface surface incident longwave radiation'
                               ' [w/m^2]',
                    'lwnet_tavg': 'surface net longwave radiation [w/m^2]',
                    'potevap_tavg': 'Potential evaporation rate [W/m^2]',
                    'psurf_f_inst': 'surface surface pressure [kPa]',
                    'qair_f_inst': 'surface near surface specific humidity [kg/kg]',
                    'qg_tavg': 'surface ground heat flux [w/m^2]',
                    'qh_tavg': 'surface sensible heat flux [w/m^2]',
                    'qle_tavg': 'surface latent heat flux [w/m^2]',
                    'qs_acc': 'storm surface runoff [kg/m^2/s]',
                    'qsb_acc': 'baseflow-groundwater runoff [kg/m^2/s]',
                    'qsm_acc': 'surface snowmelt [kg/m^2/s]',
                    'rainf_f_tavg': 'Total precipitation rate [kg/m^2/s]',
                    'rainf_tavg': 'Rain precipitation rate [kg/m^2/s]',
                    'rootmoist_inst': 'Root zone soil moisture [kg/m^2]',
                    'snowdepth_inst': 'Snow depth [m]',   
                    'swe_inst': 'surface snow water equivalent [kg/m^2]',
                    'swdown_f_tavg': 'surface surface incident shortwave radiation'
                               ' [w/m^2]',
                    'swnet_tavg': 'surface net shortwave radiation [w/m^2]',
                    'snowf_tavg': 'surface snowfall rate [kg/m^2/s]',
                    'sm0_10cm_ins': '0-10 cm underground soil moisture content'
                               ' [kg/m^2]',
                    'sm10_40cm_ins': '10-40 cm underground soil moisture content'
                               ' [kg/m^2]',
                    'sm40_100cm_ins': '40-100 cm underground soil moisture content'
                               ' [kg/m^2]',
                    'sm100_200cm_ins': '100-200 cm underground soil moisture content'
                               ' [kg/m^2]',
                    'st0_10cm_ins': '0-10 cm underground soil temperature [k]',
                    'st10_40cm_ins': '10-40 cm underground soil temperature [k]',
                    'st40_100cm_ins': '40-100 cm underground soil temperature [k]',
                    'st100_200cm_ins': '100-200 cm underground soil temperature [k]',
                    'tair_f_inst': 'surface near surface air temperature [k]',
                    'wind_f_inst': 'surface near surface wind speed [m/s]',
                    'tveg_tavg' : 'transpiration [w/m^2]'}
    factors = {'albedo_inst': 1,
               'avgsurft_inst': -273.15,
               'canopint_inst': 1,
               'ecanop_tavg': 1,
               'esoil_tavg': 1,                  
               'evap_tavg': 86400,# !!! delen door uren/24
               'lwdown_f_tavg': 1,
               'lwnet_tavg': 1,
               'potevap_tavg': 1,
               'psurf_f_inst': 0.001,
               'qair_f_inst': 1,
               'qg_tavg': 1,
               'qh_tavg': 1,
               'qle_tavg': 1,
               'qs_acc': 86400,# !!! delen door uren/24
               'qsb_acc': 86400,# !!! delen door uren/24
               'qsm_acc': 86400,# !!! delen door uren/24
               'rainf_f_tavg': 86400, # !!! delen door uren/24
               'rainf_tavg': 86400,               
               'rootmoist_inst': 1,
               'snowdepth_inst': 1,   
               'swe_inst': 1,
               'swdown_f_tavg': 1,
               'swnet_tavg': 1,
               'snowf_tavg': 1,
               'sm0_10cm_ins': 1,
               'sm10_40cm_ins': 1,
               'sm40_100cm_ins': 1,
               'sm100_200cm_ins': 1,
               'st0_10cm_ins': -273.15,
               'st10_40cm_ins': -273.15,
               'st40_100cm_ins': -273.15,
               'st100_200cm_ins': -273.15,
               'tair_f_inst': -273.15,
               'wind_f_inst': 1,
               'tveg_tavg' : 1}
    types = {'albedo_inst': 'state',
             'avgsurft_inst': 'state',
             'canopint_inst': 'state',
             'ecanop_tavg': 'state',
             'esoil_tavg': 'state',   
             'evap_tavg': 'flux',
             'lwdown_f_tavg': 'state',
             'lwnet_tavg': 'state',
             'potevap_tavg': 'state',             
             'psurf_f_inst': 'state',
             'qair_f_inst': 'state',
             'qg_tavg': 'state',
             'qh_tavg': 'state',
             'qle_tavg': 'state',
             'qs_acc': 'flux',
             'qsb_acc': 'flux',
             'qsm_acc': 'flux',
             'rainf_f_tavg': 'flux',
             'rainf_tavg': 'flux',    
             'rootmoist_inst': 'state',
             'snowdepth_inst': 'state',                
             'swe_inst': 'state',
             'swdown_f_tavg': 'state',
             'swnet_tavg': 'state',
             'snowf_tavg': 'state',
             'sm0_10cm_ins': 'state',
             'sm10_40cm_ins': 'state',
             'sm40_100cm_ins': 'state',
             'sm100_200cm_ins': 'state',
             'st0_10cm_ins': 'state',
             'st10_40cm_ins': 'state',
             'st40_100cm_ins': 'state',
             'st100_200cm_ins': 'state',
             'tair_f_inst': 'state',
             'wind_f_inst': 'state',
             'tveg_tavg' : 'state'}

    def __init__(self, step):
        if step == 'three_hourly':
            self.units = {'albedo_inst': 'Percentage',
                          'avgsurft_inst': 'C',
                          'canopint_inst': 'mm',
                          'ecanop_tavg': 'W-m-2',
                          'esoil_tavg': 'W-m-2',                          
                          'evap_tavg': 'mm-3hour-1',
                          'lwdown_f_tavg': 'W-m-2',
                          'lwnet_tavg': 'W-m-2',
                          'potevap_tavg': 'W-m-2',   
                          'psurf_f_inst': 'kpa',
                          'qair_f_inst': 'kg-kg',
                          'qg_tavg': 'W-m-2',
                          'qh_tavg': 'W-m-2',
                          'qle_tavg': 'W-m-2',
                          'qs_acc': 'mm-3hour-1',
                          'qsb_acc': 'mm-3hour-1',
                          'qsm_acc': 'mm-3hour-1',
                          'rainf_f_tavg': 'mm-3hour-1',
                          'rainf_tavg':  'mm-3hour-1',   
                          'rootmoist_inst': 'mm',
                          'snowdepth_inst': 'm',                             
                          'swe_inst': 'mm',
                          'swdown_f_tavg': 'W-m-2',
                          'swnet_tavg': 'W-m-2',
                          'snowf_tavg': 'mm',
                          'sm0_10cm_ins': 'kg-m-2',
                          'sm10_40cm_ins': 'kg-m-2',
                          'sm40_100cm_ins': 'kg-m-2',
                          'sm100_200cm_ins': 'kg-m-2',
                          'st0_10cm_ins': 'C',
                          'st10_40cm_ins': 'C',
                          'st40_100cm_ins': 'C',
                          'st100_200cm_ins': 'C',
                          'tair_f_inst': 'C',
                          'wind_f_inst': 'm-s-1',
                          'tveg_tavg' : 'W-m-2'}
        elif step == 'daily':
            self.units = {'albedo_inst': 'Percentage',
                          'avgsurft_inst': 'C',
                          'canopint_inst': 'mm',
                          'ecanop_tavg': 'W-m-2',
                          'esoil_tavg': 'W-m-2',                           
                          'evap_tavg': 'mm-day-1',
                          'lwdown_f_tavg': 'W-m-2',
                          'lwnet_tavg': 'W-m-2',
                          'potevap_tavg': 'W-m-2',   
                          'psurf_f_inst': 'kpa',
                          'qair_f_inst': 'kg-kg',
                          'qg_tavg': 'W-m-2',
                          'qh_tavg': 'W-m-2',
                          'qle_tavg': 'W-m-2',
                          'qs_acc': 'mm-day-1',
                          'qsb_acc': 'mm-day-1',
                          'qsm_acc': 'mm-day-1',
                          'rainf_f_tavg': 'mm-day-1',
                          'rainf_tavg':  'mm-day-1',                             
                          'rootmoist_inst': 'mm',
                          'snowdepth_inst': 'm',  
                          'swe_inst': 'mm',
                          'swdown_f_tavg': 'W-m-2',
                          'swnet_tavg': 'W-m-2',
                          'snowf_tavg': 'mm',
                          'sm0_10cm_ins': 'kg-m-2',
                          'sm10_40cm_ins': 'kg-m-2',
                          'sm40_100cm_ins': 'kg-m-2',
                          'sm100_200cm_ins': 'kg-m-2',
                          'st0_10cm_ins': 'C',
                          'st10_40cm_ins': 'C',
                          'st40_100cm_ins': 'C',
                          'st100_200cm_ins': 'C',
                          'tair_f_inst': 'C',
                          'wind_f_inst': 'm-s-1',
                          'tveg_tavg' : 'W-m-2'}
        elif step == 'monthly':
            self.units = {'albedo_inst': 'Percentage',
                          'avgsurft_inst': 'C',
                          'canopint_inst': 'mm',
                          'ecanop_tavg': 'W-m-2',
                          'esoil_tavg': 'W-m-2',                           
                          'evap_tavg': 'mm-month-1',
                          'lwdown_f_tavg': 'W-m-2',
                          'lwnet_tavg': 'W-m-2',
                          'potevap_tavg': 'W-m-2', 
                          'psurf_f_inst': 'kpa',
                          'qair_f_inst': 'kg-kg',
                          'qg_tavg': 'W-m-2',
                          'qh_tavg': 'W-m-2',
                          'qle_tavg': 'W-m-2',
                          'qs_acc': 'mm-month-1',
                          'qsb_acc': 'mm-month-1',
                          'qsm_acc': 'mm-month-1',
                          'rainf_f_tavg': 'mm-month-1',
                          'rainf_tavg':  'mm-month-1', 
                          'rootmoist_inst': 'mm',
                          'snowdepth_inst': 'm',                            
                          'swe_inst': 'mm',
                          'swdown_f_tavg': 'W-m-2',
                          'swnet_tavg': 'W-m-2',
                          'snowf_tavg': 'mm',
                          'sm0_10cm_ins': 'kg-m-2',
                          'sm10_40cm_ins': 'kg-m-2',
                          'sm40_100cm_ins': 'kg-m-2',
                          'sm100_200cm_ins': 'kg-m-2',
                          'st0_10cm_ins': 'C',
                          'st10_40cm_ins': 'C',
                          'st40_100cm_ins': 'C',
                          'st100_200cm_ins': 'C',
                          'tair_f_inst': 'C',
                          'wind_f_inst': 'm-s-1',
                          'tveg_tavg' : 'W-m-2'}
        else:
            raise KeyError("The input time step is not supported")


class VariablesInfo_CLSM:
    
    """
    This class contains the information about the GLDAS variables
    """
ACond_tavg = Aerodynamic conductance (m s-1)
 = Average surface skin temperature (K)
 = Plant canopy surface water (kg m-2)
ECanop_tavg = Canopy water evaporation (kg m-2 s-1)
ESoil_tavg = Direct evaporation from bare soil (kg m-2 s-1)
 = Evapotranspiration (kg m-2 s-1)
EvapSnow_tavg = Snow evaporation (kg m-2 s-1)
GWS_tavg = Ground water storage (mm)
 = Downward longwave radiation flux (W m-2)
 = Net longwave radiation flux (W m-2)
 = Surface air pressure (Pa)
 = Specific humidity (kg kg-1)
 = Ground heat flux (W m-2)
 = Sensible heat net flux (W m-2)
 = Latent heat net flux (W m-2)
 = Storm surface runoff (kg m-2 s-1)
 = Baseflow-groundwater runoff (kg m-2 s-1)
 = Snow melt (kg m-2 s-1)
 = Total precipitation rate (kg m-2 s-1)
 = Rain precipitation rate (kg m-2 s-1)
SnowDepth_tavg = Snow depth (m)
 = Snow precipitation rate (kg m-2 s-1)
SnowT_tavg = Snow surface temperature (K)
 = Profile soil moisture (kg m-2)
 = Root zone soil moisture (kg m-2)
 = Surface soil moisture (kg m-2)
 = Downward shortwave radiation flux (W m-2)
 = Snow depth water equivalent (kg m-2)
 = Net shortwave radiation flux (W m-2)
 = Air temperature (K)
 = Transpiration (kg m-2 s-1)
TWS_tavg = Terrestrial water storage (mm)
 = Wind speed (m s-1)

    parameters = {'avgsurft_tavg': 'AvgSurfT_tavg',
             'canopint_tavg': 'CanopInt_tavg',
             'evap_tavg': 'Evap_tavg',
             'lwdown_f_tavg': 'LWdown_f_tavg',
             'lwnet_tavg': 'Lwnet_tavg',
             'psurf_f_tavg': 'Psurf_f_tavg',
             'qair_f_tavg': 'Qair_f_tavg',
             'qg_tavg': 'Qg_tavg',
             'qh_tavg': 'Qh_tavg',
             'qle_tavg': 'Qle_tavg',
             'qs_tavg': 'Qs_tavg',
             'qsb_tavg': 'Qsb_tavg',
             'qsm_tavg': 'Qsm_tavg',
             'rainf_f_tavg': 'Rainf_f_tavg',
             'rainf_tavg': 'Rainf_tavg',
             'swe_tavg': 'SWE_tavg',
             'swdown_f_tavg': 'SWdown_f_tavg',
             'swnet_tavg': 'SWnet_tavg',
             'snowf_tavg': 'Snowf_tavg',
             'soilmoist_s_tav': 'SoilMoist_S_tavg',
             'soilmoist_rz_ta': 'SoilMoist_RZ_tavg',
             'soilmoist_p_tav': 'SoilMoist_P_tavg',
             'tair_f_tavg': 'Tair_f_tavg',
             'wind_f_tavg': 'Wind_f_tavg',
             'tveg_tavg' : 'TVeg_tavg'}  
    
    
    names = {'avgsurft_tavg': 'SurfaceTemperature',
             'canopint_tavg': 'TotCanopyWaterStorage',
             'evap_tavg': 'ET',
             'lwdown_f_tavg': 'LWdown',
             'lwnet_tavg': 'LWnet',
             'psurf_f_tavg': 'P',
             'qair_f_tavg': 'Hum',
             'qg_tavg': 'G',
             'qh_tavg': 'H',
             'qle_tavg': 'LE',
             'qs_tavg': 'Rsur',
             'qsb_tavg': 'Rsubsur',
             'qsm_tavg': 'SnowMelt',
             'rainf_f_tavg': 'P',
             'rainf_tavg': 'Prain',
             'swe_tavg': 'SnowWaterEquivalent',
             'swdown_f_tavg': 'SWdown',
             'swnet_tavg': 'SWnet',
             'snowf_tavg': 'Snow',
             'soilmoist_s_tav': 'SoilMoisturSurface',
             'soilmoist_rz_ta': 'SoilMoistureRootZone',
             'soilmoist_p_tav': 'SoilMoistureProfile',
             'tair_f_tavg': 'Tair',
             'wind_f_tavg': 'W',
             'tveg_tavg' : 'Transpiration'}
    
    descriptions = {'avgsurft_tavg': 'surface average surface temperature [k]',
                    'canopint_tavg': 'surface plant canopy surface water [kg/m^2]',
                    'evap_tavg': 'surface total evapotranspiration [kg/m^2/s]',
                    'lwdown_f_tavg': 'surface surface incident longwave radiation'
                               ' [w/m^2]',
                    'lwnet_tavg': 'surface net longwave radiation [w/m^2]',
                    'psurf_f_tavg': 'surface surface pressure [kPa]',
                    'qair_f_tavg': 'surface near surface specific humidity [kg/kg]',
                    'qg_tavg': 'surface ground heat flux [w/m^2]',
                    'qh_tavg': 'surface sensible heat flux [w/m^2]',
                    'qle_tavg': 'surface latent heat flux [w/m^2]',
                    'qs_tavg': 'storm surface runoff [kg/m^2/s]',
                    'qsb_tavg': 'baseflow-groundwater runoff [kg/m^2/s]',
                    'qsm_tavg': 'surface snowmelt [kg/m^2/s]',
                    'rainf_f_tavg': 'surface rainfall rate [kg/m^2/s]',
                    'rainf_tavg': 'Rain precipitation rate [kg/m^2/s]',
                    'swe_tavg': 'surface snow water equivalent [kg/m^2]',
                    'swdown_f_tavg': 'surface surface incident shortwave radiation'
                               ' [w/m^2]',
                    'swnet_tavg': 'surface net shortwave radiation [w/m^2]',
                    'snowf_tavg': 'surface snowfall rate [kg/m^2/s]',
                    'soilmoist_s_tav': 'surface soil moisture [kg/m^2]',
                    'soilmoist_rz_ta': 'root zone soil moisture [kg/m^2]',
                    'soilmoist_p_tav': 'profile soil moisture [kg/m^2]',
                    'tair_f_tavg': 'surface near surface air temperature [k]',
                    'wind_f_tavg': 'surface near surface wind speed [m/s]',
                    'tveg_tavg' : 'transpiration [w/m^2]'}
    
    factors = {'avgsurft_tavg': -273.15,
               'canopint_tavg': 1,
               'evap_tavg': 86400,
               'lwdown_f_tavg': 1,
               'lwnet_tavg': 1,
               'psurf_f_tavg': 0.001,
               'qair_f_tavg': 1,
               'qg_tavg': 1,
               'qh_tavg': 1,
               'qle_tavg': 1,
               'qs_tavg': 86400,
               'qsb_tavg': 86400,
               'qsm_tavg': 86400,
               'rainf_f_tavg': 86400,
               'rainf_tavg': 86400,
               'swe_tavg': 1,
               'swdown_f_tavg': 1,
               'swnet_tavg': 1,
               'snowf_tavg': 1,
               'soilmoist_s_tav': 1,
               'soilmoist_rz_ta': 1,
               'soilmoist_p_tav': 1,
               'tair_f_tavg': -273.15,
               'wind_f_tavg': 0.75,
               'tveg_tavg' : 1}
    types = {'avgsurft_tavg': 'state',
             'canopint_tavg': 'state',
             'evap_tavg': 'flux',
             'lwdown_f_tavg': 'state',
             'lwnet_tavg': 'state',
             'psurf_f_tavg': 'state',
             'qair_f_tavg': 'state',
             'qg_tavg': 'state',
             'qh_tavg': 'state',
             'qle_tavg': 'state',
             'qs_tavg': 'flux',
             'qsb_tavg': 'flux',
             'qsm_tavg': 'flux',
             'rainf_f_tavg': 'flux',
             'rainf_tavg': 'flux',
             'swe_tavg': 'state',
             'swdown_f_tavg': 'state',
             'swnet_tavg': 'state',
             'snowf_tavg': 'state',
             'soilmoist_s_tav': 'state',
             'soilmoist_rz_ta': 'state',
             'soilmoist_p_tav': 'state',
             'tair_f_tavg': 'state',
             'wind_f_tavg': 'state',
             'tveg_tavg' : 'state'}

    def __init__(self, step):
        if step == 'three_hourly':
            self.units = {'avgsurft_tavg': 'C',
                          'canopint_tavg': 'mm',
                          'evap_tavg': 'mm-3hour-1',
                          'lwdown_f_tavg': 'W-m-2',
                          'lwnet_tavg': 'W-m-2',
                          'psurf_f_tavg': 'kpa',
                          'qair_f_tavg': 'kg-kg',
                          'qg_tavg': 'W-m-2',
                          'qh_tavg': 'W-m-2',
                          'qle_tavg': 'W-m-2',
                          'qs_tavg': 'mm-3hour-1',
                          'qsb_tavg': 'mm-3hour-1',
                          'qsm_tavg': 'mm-3hour-1',
                          'rainf_f_tavg': 'mm-3hour-1',
                          'rainf_tavg': 'mm-3hour-1',
                          'swe_tavg': 'mm',
                          'swdown_f_tavg': 'W-m-2',
                          'swnet_tavg': 'W-m-2',
                          'snowf_tavg': 'mm',
                          'soilmoist_s_tav': 'mm',
                          'soilmoist_rz_ta': 'mm',
                          'soilmoist_p_tav': 'mm',                          
                          'tair_f_tavg': 'C',
                          'wind_f_tavg': 'm-s-1',
                          'tveg_tavg': 'mm'}
        elif step == 'daily':
            self.units = {'avgsurft_tavg': 'C',
                          'canopint_tavg': 'mm',
                          'evap_tavg': 'mm-day-1',
                          'lwdown_f_tavg': 'W-m-2',
                          'lwnet_tavg': 'W-m-2',
                          'psurf_f_tavg': 'kpa',
                          'qair_f_tavg': 'kg-kg',
                          'qg_tavg': 'W-m-2',
                          'qh_tavg': 'W-m-2',
                          'qle_tavg': 'W-m-2',
                          'qs_tavg': 'mm-day-1',
                          'qsb_tavg': 'mm-day-1',
                          'qsm_tavg': 'mm-day-1',
                          'rainf_f_tavg': 'mm-day-1',
                          'rainf_tavg': 'mm-day-1',
                          'swe_tavg': 'mm',
                          'swdown_f_tavg': 'W-m-2',
                          'swnet_tavg': 'W-m-2',
                          'snowf_tavg': 'mm',
                          'soilmoist_s_tav': 'mm',
                          'soilmoist_rz_ta': 'mm',
                          'soilmoist_p_tav': 'mm',                             
                          'tair_f_tavg': 'C',
                          'wind_f_tavg': 'm-s-1',
                          'tveg_tavg': 'mm'}
        elif step == 'monthly':
            self.units = {'avgsurft_tavg': 'C',
                          'canopint_tavg': 'mm',
                          'evap_tavg': 'mm-month-1',
                          'lwdown_f_tavg': 'W-m-2',
                          'lwnet_tavg': 'W-m-2',
                          'psurf_f_tavg': 'kpa',
                          'qair_f_tavg': 'kg-kg',
                          'qg_tavg': 'W-m-2',
                          'qh_tavg': 'W-m-2',
                          'qle_tavg': 'W-m-2',
                          'qs_tavg': 'mm-month-1',
                          'qsb_tavg': 'mm-month-1',
                          'qsm_tavg': 'mm-month-1',
                          'rainf_f_tavg': 'mm-month-1',
                          'rainf_tavg': 'mm-month-1',                          
                          'swe_tavg': 'mm',
                          'swdown_f_tavg': 'W-m-2',
                          'swnet_tavg': 'W-m-2',
                          'snowf_tavg': 'mm',
                          'soilmoist_s_tav': 'mm',
                          'soilmoist_rz_ta': 'mm',
                          'soilmoist_p_tav': 'mm',                             
                          'tair_f_tavg': 'C',
                          'wind_f_tavg': 'm-s-1',
                          'tveg_tavg': 'mm'}
        else:
            raise KeyError("The input time step is not supported")













https://hydro1.gesdisc.eosdis.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fdata%2FGLDAS%2FGLDAS_CLSM025_D.2.0%2F1948%2F01%2FGLDAS_CLSM025_D.A19480105.020.nc4&FORMAT=bmM0Lw&BBOX=37.969%2C-7.031%2C61.172%2C19.688&LABEL=GLDAS_CLSM025_D.A19480105.020.nc4.SUB.nc4&SHORTNAME=GLDAS_CLSM025_D&SERVICE=L34RS_LDAS&VERSION=1.02&DATASET_VERSION=2.0
https://hydro1.gesdisc.eosdis.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fdata%2FGLDAS%2FGLDAS_NOAH025_M.2.1%2F2001%2FGLDAS_NOAH025_M.A200101.021.nc4&FORMAT=Y29nLw&BBOX=25.313%2C-121.641%2C59.063%2C-82.969&LABEL=GLDAS_NOAH025_M.A200101.021.nc4.SUB.nc4&SHORTNAME=GLDAS_NOAH025_M&SERVICE=L34RS_LDAS&VERSION=1.02&DATASET_VERSION=2.1&VARIABLES=Psurf_f_inst%2CWind_f_inst%2CTair_f_inst
https://hydro1.gesdisc.eosdis.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fdata%2FGLDAS%2FGLDAS_NOAH025_M.2.1%2F2000%2FGLDAS_NOAH025_M.A200001.021.nc4&FORMAT=bmM0Lw&BBOX=19.688%2C-119.531%2C50.625%2C-86.484&LABEL=GLDAS_NOAH025_M.A200001.021.nc4.SUB.nc4&SHORTNAME=GLDAS_NOAH025_M&SERVICE=L34RS_LDAS&VERSION=1.02&DATASET_VERSION=2.1&VARIABLES=Psurf_f_inst%2CTair_f_inst%2CWind_f_inst
https://hydro1.gesdisc.eosdis.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fdata%2FGLDAS%2FGLDAS_NOAH025_M.2.1%2F2001%2FGLDAS_NOAH025_M.A200101.021.nc4&FORMAT=Y29nLw&BBOX=25.313%2C-121.641%2C59.063%2C-82.969&LABEL=GLDAS_NOAH025_M.A200101.021.nc4.SUB.nc4&SHORTNAME=GLDAS_NOAH025_M&SERVICE=L34RS_LDAS&VERSION=1.02&DATASET_VERSION=2.1&VARIABLES=Psurf_f_inst%2CTair_f_inst%2CWind_f_inst
https://hydro1.gesdisc.eosdis.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fdata%2FGLDAS%2FGLDAS_NOAH025_3H.2.1%2F2000%2F001%2FGLDAS_NOAH025_3H.A20000101.0300.021.nc4&FORMAT=bmM0Lw&BBOX=-60%2C-180%2C90%2C180&LABEL=GLDAS_NOAH025_3H.A20000101.0300.021.nc4.SUB.nc4&SHORTNAME=GLDAS_NOAH025_3H&SERVICE=L34RS_LDAS&VERSION=1.02&DATASET_VERSION=2.1&VARIABLES=Albedo_inst%2CLWdown_f_tavg%2CWind_f_inst
https://hydro1.gesdisc.eosdis.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fdata%2FGLDAS%2FGLDAS_NOAH025_M.2.1%2F2000%2FGLDAS_NOAH025_M.A200001.021.nc4&FORMAT=Y29nLw&BBOX=25.313%2C-121.641%2C59.063%2C-82.969&LABEL=GLDAS_NOAH025_M.A200001.021.nc4.SUB.tif&SHORTNAME=GLDAS_NOAH025_M&SERVICE=L34RS_LDAS&VERSION=1.02&DATASET_VERSION=2.1&VARIABLES=Wind_f_inst
https://hydro1.gesdisc.eosdis.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fdata%2FGLDAS%2FGLDAS_NOAH025_M.2.1%2F2003%2FGLDAS_NOAH025_M.A200303.021.nc4&FORMAT=Y29nLw&BBOX=25.313%2C-121.641%2C59.063%2C-82.969&LABEL=GLDAS_NOAH025_M.A200303.021.nc4.SUB.tif&SHORTNAME=GLDAS_NOAH025_M&SERVICE=L34RS_LDAS&VERSION=1.02&DATASET_VERSION=2.1&VARIABLES=Wind_f_inst
https://hydro1.gesdisc.eosdis.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fdata%2FGLDAS%2FGLDAS_NOAH025_M.2.0%2F2001%2FGLDAS_NOAH025_M.A200101.020.nc4&FORMAT=Y29nLw&BBOX=25.313%2C-121.641%2C59.063%2C-82.969&LABEL=GLDAS_NOAH025_M.A200101.020.nc4.SUB.tif&SHORTNAME=GLDAS_NOAH025_M&SERVICE=L34RS_LDAS&VERSION=1.02&DATASET_VERSION=2.0&VARIABLES=Wind_f_inst
https://hydro1.gesdisc.eosdis.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fdata%2FGLDAS%2FGLDAS_NOAH025_M.2.1%2F2001%2FGLDAS_NOAH025_M.A200101.021.nc4&FORMAT=Y29nLw&BBOX=25.313%2C-121.641%2C59.063%2C-82.969&LABEL=GLDAS_NOAH025_M.A200101.021.nc4.SUB.tif&SHORTNAME=GLDAS_NOAH025_M&SERVICE=L34RS_LDAS&VERSION=1.02&DATASET_VERSION=2.1&VARIABLES=Wind_f_inst
https://hydro1.gesdisc.eosdis.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fdata%2FGLDAS%2FGLDAS_NOAH025_3H.2.1%2F2000%2F001%2FGLDAS_NOAH025_3H.A20000101.1200.021.nc4&FORMAT=bmM0Lw&BBOX=-60%2C-180%2C90%2C180&LABEL=GLDAS_NOAH025_3H.A20000101.1200.021.nc4.SUB.nc4&SHORTNAME=GLDAS_NOAH025_3H&SERVICE=L34RS_LDAS&VERSION=1.02&DATASET_VERSION=2.1&VARIABLES=Wind_f_inst

https://hydro1.gesdisc.eosdis.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fdata%2FGLDAS%2FGLDAS_NOAH025_3H.2.0%2F1948%2F001%2FGLDAS_NOAH025_3H.A19480101.2100.020.nc4&FORMAT=Y29nLw&BBOX=30.938%2C-104.062%2C51.328%2C-71.016&LABEL=GLDAS_NOAH025_3H.A19480101.2100.020.nc4.SUB.tif&SHORTNAME=GLDAS_NOAH025_3H&SERVICE=L34RS_LDAS&VERSION=1.02&DATASET_VERSION=2.0&VARIABLES=Wind_f_inst
https://hydro1.gesdisc.eosdis.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fdata%2FGLDAS%2FGLDAS_NOAH025_3H.2.1%2F2000%2F001%2FGLDAS_NOAH025_3H.A20000101.0900.021.nc4&FORMAT=dGlmLw&BBOX=-60%2C-180%2C90%2C180&LABEL=GLDAS_NOAH025_3H.A20000101.0900.021.nc4.SUB.tif&SHORTNAME=GLDAS_NOAH025_3H&SERVICE=L34RS_LDAS&VERSION=1.02&DATASET_VERSION=2.1&VARIABLES=Wind_f_inst


# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Contact: timhessels@hotmail.com
Repository: https://github.com/TimHessels/watertools
Module: Collect


Description:
This module contains scripts used to download Level 1 data (data directly from web).

Products                      Dates                             Password
ALEXI (daily)                 2005/01/01-2016/12/31             WA+ FTP
ALEXI (monthly)               2005/01/01-2016/12/31             WA+ FTP                         
ASCAT (daily)                 2007/01/01-now                    VITO
CFSR (daily)                  1979/01/01-now                    -
CHIRPS (daily)                1981/01/01-now                    -
CHIRPS (monthly)              1981/01/01-now                    -
CMRSET (monthly)              2000/01/01-2012/12/31             WA+ FTP
DEM                           -                                 -
ECMWF                         1979/01/01-now                    ECMWF_API
ESACCI                        -                                 -
ETmonitor (monthly)           2008/01/01-2013/12/31             WA+ FTP    
GLDAS                         2000/01/01-now                    NASA
GLEAM (daily)                 2007/01/01-2017/12/31             GLEAM
GLEAM (monthly)               2007/01/01-2017/12/31             GLEAM 
GPM (daily)                   2014/04/01-now                    NASA
GPM (monthly)                 2014/04/01-now                    NASA        
HiHydroSoil                   -                                 WA+ FTP 
JRC                           -                                 -
MCD43 (daily)                 2000/02/24-now                    NASA
MOD10 (8-daily)               2000/02/18-now                    NASA
MOD11 (daily)                 2000/02/24-now                    NASA
MOD11 (8-daily)               2000/02/18-now                    NASA
MOD12 (yearly)                2001/01/01-2013/12/31             NASA
MOD13 (16-daily)              2000/02/18-now                    NASA
MOD15 (8-daily)               2000/02/18-now                    NASA
MOD16 (8-daily)               2000/01/01-2014/12/31             NASA
MOD16 (monthly)               2000/01/01-2014/12/31             NASA
MOD17 (8-daily GPP)           2000/02/18-now                    NASA
MOD17 (yearly NPP)            2000/02/18-2015/12/31             NASA
MOD9 (daily)                  2000/02/24-now                    NASA
MSWEP (daily)                 1979/01/01-now                    MSWEP
MSWEP (monthly)               1979/01/01-now                    MSWEP
MYD13 (16-daily)              2000/02/18-now                    NASA
NLDAS                         1979/01/01-now                    NASA
RFE (daily)                   2001/01/01-now                    -
RFE (monthly)                 2001/01/01-now                    -                  
SEBS (monthly)                2000/03/01-2015/12/31             WA+ guest FTP
SSEBop (monthly)              2003/01/01-2014/10/31             WA+ FTP 
SoilGrids                     -                                 -
TRMM (daily)                  1998/01/01-2018/06/29             NASA
TRMM (monthly)                1998/01/01-2018/06/30             NASA
TWC                           -                                 WA+ guest FTP 

Examples:
from watools import Collect
help(Collect)
dir(Collect)
"""

from watertools.Collect import TRMM, GLDAS, ALEXI, CHIRPS, CHIRP, DEM, CFSR, MOD9, MOD10, MOD11, MOD12, MOD13, MOD15, MOD16, MOD17, MCD43, MCD19, MYD9, MYD11, MYD13, GLEAM, HiHydroSoil, ECMWF, RFE, JRC, TWC, ETmonitor, SEBS, SSEBop, CMRSET, MSWEP, ASCAT, SoilGrids, FEWS, PROBAV, GPM, NLDAS, S30, L30, Globcover, MERRA, GEOS, MSGCPP, ESACCI, WAPOR

__all__ = ['TRMM', 'GLDAS', 'ALEXI', 'CHIRPS', 'CHIRP', 'DEM', 'CFSR', 'MOD9', 'MOD10', 'MOD11', 'MOD12', 'MOD13', 'MOD15', 'MOD16', 'MOD17', 'MCD43', 'MCD19', 'MYD9', 'MYD11','MYD13', 'GLEAM', 'HiHydroSoil', 'ECMWF', 'RFE', 'JRC', 'TWC', 'ETmonitor', 'SEBS', 'SSEBop', 'CMRSET', 'MSWEP', 'ASCAT', 'SoilGrids', 'FEWS', 'PROBAV', 'GPM', 'NLDAS', 'S30', 'L30', 'Globcover', 'MERRA', 'GEOS', 'MSGCPP', 'ESACCI', 'WAPOR']

__version__ = '0.1'



# -*- coding: utf-8 -*-
# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Collect/GLEAM
"""

# import general python modules
import os
import numpy as np
import pandas as pd
import glob
from joblib import Parallel, delayed
import paramiko
import calendar
from netCDF4 import Dataset

# Water Accounting modules
import watertools.General.data_conversions as DC

def DownloadData(Dir, Startdate, Enddate, latlim, lonlim, Waitbar, cores, TimeCase, Product):
    """
    This function downloads GLEAM ET data

    Keyword arguments:
    Dir -- 'C:/file/to/path/'
    Startdate -- 'yyyy-mm-dd'
    Enddate -- 'yyyy-mm-dd'
    latlim -- [ymin, ymax] (values must be between -50 and 50)
    lonlim -- [xmin, xmax] (values must be between -180 and 180)
    cores -- The number of cores used to run the routine. It can be 'False'
             to avoid using parallel computing routines.
    Waitbar -- 1 (Default) will print a waitbar
    """
    # Check start and end date and otherwise set the date
    if not Startdate:
        Startdate = pd.Timestamp('2003-01-01')
    if not Enddate:
        Enddate = pd.Timestamp('2015-12-31')

    # Make an array of the days of which the ET is taken
    YearsDownloadstart = str(Startdate[0:4])
    YearsDownloadend = str(Enddate[0:4])
    Years = range(int(YearsDownloadstart),int(YearsDownloadend)+1)

    # String Parameters
    if TimeCase == 'daily':
        VarCode = '%s_GLEAM.V3.3b_mm-day-1_daily' %Product
        FTPprefix = 'data/v3.3b/'
        TimeFreq = 'D'
        Folder_name = 'Daily'

    elif TimeCase == 'monthly':
        VarCode = '%s_GLEAM.V3.3b_mm-month-1_monthly' %Product
        FTPprefix = 'data/v3.3b/'
        TimeFreq = 'M'
        Folder_name = 'Monthly'

        # Get end of month for Enddate
        monthDownloadend = str(Enddate[5:7])
        End_month = calendar.monthrange(int(YearsDownloadend),int(monthDownloadend))[1]
        Enddate = '%d-%02d-%d' %(int(YearsDownloadend),int(monthDownloadend),int(End_month))
    else:
        raise KeyError("The input time interval is not supported")

    Dates = pd.date_range(Startdate, Enddate, freq = TimeFreq)

    # Make directory for the MODIS ET data
    if Product == "ET":
        output_folder=os.path.join(Dir,'Evaporation', 'GLEAM', Folder_name)
        if not os.path.exists(output_folder):
            os.makedirs(output_folder)
    if Product == "ETpot":
        output_folder=os.path.join(Dir,'Potential_Evapotranspiration', 'GLEAM', Folder_name)
        if not os.path.exists(output_folder):
            os.makedirs(output_folder)	

    # Check variables
    if latlim[0] < -50 or latlim[1] > 50:
        print('Latitude above 50N or below 50S is not possible.'
               ' Value set to maximum')
        latlim[0] = np.max(latlim[0], -50)
        latlim[1] = np.min(lonlim[1], 50)
    if lonlim[0] < -180 or lonlim[1] > 180:
        print('Longitude must be between 180E and 180W.'
               ' Now value is set to maximum')
        lonlim[0] = np.max(latlim[0], -180)
        lonlim[1] = np.min(lonlim[1], 180)

    # Collect the data from the GLEAM webpage and returns the data and lat and long in meters of those tiles
    try:
        Collect_data(FTPprefix, Years, output_folder, Waitbar, Product)
    except:
        print("Was not able to download the file")

    # Create Waitbar
    print('\nProcess the GLEAM data')
    if Waitbar == 1:
        import watertools.Functions.Random.WaitbarConsole as WaitbarConsole
        total_amount = len(Dates)
        amount = 0
        WaitbarConsole.printWaitBar(amount, total_amount, prefix = 'Progress:', suffix = 'Complete', length = 50)

    # Pass variables to parallel function and run
    args = [output_folder, latlim, lonlim, VarCode, TimeCase, Product]
    if not cores:
        for Date in Dates:
            RetrieveData(Date, args)
            if Waitbar == 1:
                amount += 1
                WaitbarConsole.printWaitBar(amount, total_amount, prefix = 'Progress:', suffix = 'Complete', length = 50)
        results = True
    else:
        results = Parallel(n_jobs=cores)(delayed(RetrieveData)(Date, args)
                                         for Date in Dates)

    # Remove all .hdf files
    os.chdir(output_folder)
    files = glob.glob("*.nc")
    for f in files:
        os.remove(os.path.join(output_folder, f))

    return(results)

def RetrieveData(Date, args):
    """
    This function retrieves GLEAM ET data for a given date from the
    www.gleam.eu server.

    Keyword arguments:
    Date -- 'yyyy-mm-dd'
    args -- A list of parameters defined in the DownloadData function.
    """
    # Argument
    [output_folder, latlim, lonlim, VarCode, TimeCase, Product] = args

    # Adjust latlim to GLEAM dataset
    latlim1=[latlim[1]*-1, latlim[0]*-1]

    # select the spatial dataset
    Ystart=int(np.floor((latlim1[0]+90)/0.25))
    Yend=int(np.ceil((latlim1[1]+90)/0.25))
    Xstart=int(np.floor((lonlim[0]+180)/0.25))
    Xend=int(np.ceil((lonlim[1]+180)/0.25))

    Year=Date.year
    Month=Date.month
    
    if TimeCase == 'monthly':

        if Product == "ET":
            filename='E_' + str(Year) + '_GLEAM_v3.3b_MO.nc'
        if Product == "ETpot":
            filename='Ep_' + str(Year) + '_GLEAM_v3.3b_MO.nc'
        
        local_filename = os.path.join(output_folder, filename)

        f = Dataset(local_filename,mode='r')

        # defines the start and end of the month
        Datesend1=str(Date)
        Datesend2=Datesend1.replace(Datesend1[8:10],"01")
        Datesend3=Datesend2[0:10]
        Datesend4=Datesend1[0:10]
        Datestart = pd.date_range(Datesend3,Datesend4,freq = 'MS')

        # determine the DOY-1 and DOYend (those are use to define the temporal boundaries of the yearly data)
        DOY=int(Datestart[0].strftime('%j'))
        DOYend=int(Date.strftime('%j'))
        DOYDownload=DOY-1
        Day = 1

        if Product == "ET":
            Data = f.variables['E'][DOYDownload:DOYend,Xstart:Xend,Ystart:Yend]
        if Product == "ETpot":
            Data = f.variables['Ep'][DOYDownload:DOYend,Xstart:Xend,Ystart:Yend]
           
        data=np.array(Data)
        f.close()

        # Sum ET data in time and change the no data value into -999
        dataSum=sum(data,1)
        dataSum[dataSum<-100]=-999.000
        dataCor=np.swapaxes(dataSum,0,1)

    if TimeCase == 'daily':
        
        if Product == "ET":
            filename='E_' + str(Year) + '_GLEAM_v3.3b.nc'
        if Product == "ETpot":
            filename='Ep_' + str(Year) + '_GLEAM_v3.3b.nc'
        
        local_filename = os.path.join(output_folder, filename)

        f = Dataset(local_filename,mode='r')
        
        
        Day = Date.day

        # Define the DOY, DOY-1 is taken from the yearly dataset
        DOY=int(Date.strftime('%j'))
        DOYDownload=DOY-1
        
        if Product == "ET":
            Data = f.variables['E'][DOYDownload,Xstart:Xend,Ystart:Yend]
        if Product == "ETpot":
            Data = f.variables['Ep'][DOYDownload,Xstart:Xend,Ystart:Yend]        
        data=np.array(Data)
        f.close()

        data[data<-100]=-999.000
        dataCor=np.swapaxes(data,0,1)

    # The Georeference of the map
    geo_in=[lonlim[0], 0.25, 0.0, latlim[1], 0.0, -0.25]

    # Name of the map
    dataset_name=VarCode + '_' + str(Year) + '.' + str(Month).zfill(2)  + '.' + str(Day).zfill(2) + '.tif'
    output_file=os.path.join(output_folder, dataset_name)

    # save data as tiff file
    DC.Save_as_tiff(name=output_file, data=dataCor, geo=geo_in, projection="WGS84")

    return True


def Collect_data(FTPprefix,Years,output_folder, Waitbar, Product):
    '''
    This function downloads all the needed GLEAM files from hydras.ugent.be as a nc file.

    Keywords arguments:
    FTPprefix -- FTP path to the GLEAM data
    Date -- 'yyyy-mm-dd'
    output_folder -- 'C:/file/to/path/'
    '''
    # account of the SFTP server (only password is missing)
    server='hydras.ugent.be'
    portnumber=2225

    username, password = watertools.Functions.Random.Get_Username_PWD.GET("GLEAM")

    # Create Waitbar
    print('\nDownload GLEAM data')
    if Waitbar == 1:
        import watertools.Functions.Random.WaitbarConsole as WaitbarConsole
        total_amount2 = len(Years)
        amount2 = 0
        WaitbarConsole.printWaitBar(amount2, total_amount2, prefix = 'Progress:', suffix = 'Complete', length = 50)


    for year in Years:
        directory = os.path.join(FTPprefix, '%d' %year)
        ssh=paramiko.SSHClient()
        ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        ssh.connect(server, port=portnumber, username=username, password=password)
        ftp=ssh.open_sftp()
        ftp.chdir(directory)
        
        if Product == "ET":
             filename='E_' + str(year) + '_GLEAM_v3.3b.nc'
        if Product == "ETpot":
             filename='Ep_' + str(year) + '_GLEAM_v3.3b.nc'
        local_filename = os.path.join(output_folder, filename)

        if not os.path.exists(local_filename):
            ftp.get(filename, local_filename)

        if Waitbar == 1:
            amount2 += 1
            WaitbarConsole.printWaitBar(amount2, total_amount2, prefix = 'Progress:', suffix = 'Complete', length = 50)


    ftp.close()
    ssh.close()

    return()


# -*- coding: utf-8 -*-
"""
Created on Tue Aug 07 14:43:01 2018

@author: tih
"""
import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="watertools",
    version="0.0.39",
    author="Tim Hessels",
    author_email="timhessels@hotmail.com",
    description="Tools for data collecting and data processing of remote sensed data.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/TimHessels/watertools",
    packages=setuptools.find_packages(),
    classifiers=(
	    "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ),
)


# -*- coding: utf-8 -*-
"""
WaterSat
author: Tim Martijn Hessels
Created on Sun Feb 10 18:26:30 2019
"""
import datetime
import pandas as pd
import numpy as np
import os
import urllib
from osgeo import gdal

#S2_tile = "10SGE"
#output_folder = "F:\Project_Jain\Case_California\Input_Data\S30"
#Startdate = "2018-03-01"
#Enddate = "2018-10-31"

def DownloadData(Dir, Startdate, Enddate, S2_tile):

    # Import watertools modules
    import watertools.General.data_conversions as DC
    
    # Define the dates
    Dates = pd.date_range(Startdate, Enddate, freq = "D")
    
    # Create output folder
    output_folder_end = os.path.join(Dir, S2_tile)
    if not os.path.exists(output_folder_end):
        os.makedirs(output_folder_end)
    
    # Loop over the days
    for Date in Dates:
        
        # Get the datum
        doy = int(Date.dayofyear)
        year = Date.year
        
        try:
            
            # Create the right url
            url = "https://hls.gsfc.nasa.gov/data/v1.4/S30/%s/%s/%s/%s/%s/HLS.S30.T%s.%s%03d.v1.4.hdf" % (year, S2_tile[0:2], S2_tile[2:3], S2_tile[3:4], S2_tile[4:5],S2_tile, year, doy)
            filename_out = os.path.join(output_folder_end, "HLS.S30.T%s.%s%03d.v1.4.hdf" % (S2_tile, year, doy))

            # Download the data
            urllib.request.urlretrieve(url, filename=filename_out)
            
            # Create a folder for the end results 
            folder_tile = os.path.join(output_folder_end, "HLS_S30_T%s_%s%03d_v1_4" % (S2_tile, year, doy))
            if not os.path.exists(folder_tile):
                os.makedirs(folder_tile)        
            
            # Write the hdf file in tiff files and store it in the folder
            dataset = gdal.Open(filename_out)
            sdsdict = dataset.GetMetadata('SUBDATASETS')
            
            for Band in range(1,15):
                dest = gdal.Open(sdsdict["SUBDATASET_%d_NAME" %Band])
                Array = dest.GetRasterBand(1).ReadAsArray()
                if Band < 9.:
                    Array = Array * 0.0001
                    Array[Array == -0.1] = -9999.
                    Band_name = "B%02d" %(Band)
                if Band == 9.:
                    Band_name = "B8A"
                    Array = Array * 0.0001
                    Array[Array == -0.1] = -9999.
                if (Band >= 10. and Band < 14.):
                    Band_name = "B%02d" %(Band - 1)   
                    Array = Array * 0.0001
                    Array[Array == -0.1] = -9999.
                if Band == 14.:
                    Array[Array == 255] = -9999.
                    Band_name = "QC"
                
                meta = dataset.GetMetadata()
                ulx = int(meta["ULX"])
                uly = int(meta["ULY"])   
                size = int(meta["SPATIAL_RESOLUTION"]) 
                projection = int(meta["HORIZONTAL_CS_CODE"].split(":")[-1])
                
                time_string = meta["SENSING_TIME"].split(";")[0]

                Time = datetime.datetime.strptime(time_string[:-5],"%Y-%m-%dT%H:%M:%S")
                hour = int(Time.hour)
                minute = int(Time.minute)
                geo = tuple([ulx, size, 0, uly, 0, -size])
                DC.Save_as_tiff(os.path.join(folder_tile, "HLS_S30_T%s_%s%03d_H%02dM%02d_%s.tif" % (S2_tile, year, doy, hour, minute, Band_name)), Array, geo, projection)
                     
        except:
            pass
        
    return()


# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Collect/DEM
"""
import os
from watertools.Collect.DEM.DataAccess_SRTM import DownloadData
import sys


def main(Dir, latlim, lonlim, Waitbar = 1):
    """
    Downloads HydroSHED data from http://srtm.csi.cgiar.org/download

    this data includes a Digital Elevation Model (DEM)
    The spatial resolution is 90m (3s)

    The following keyword arguments are needed:
    Dir -- 'C:/file/to/path/'
    latlim -- [ymin, ymax]
    lonlim -- [xmin, xmax]
    Waitbar -- '1' if you want a waitbar (Default = 1)
    """

    # Create directory if not exists for the output
    output_folder = os.path.join(Dir, 'SRTM', 'DEM')
    if not os.path.exists(output_folder):
        os.makedirs(output_folder)

    # Define the output map and create this if not exists
    nameEnd = os.path.join(output_folder, 'DEM_SRTM_m_3s.tif')

    if not os.path.exists(nameEnd):

        # Create Waitbar
        if Waitbar == 1:
            print('\nDownload SRTM altitude map with a resolution of 3s')
            import watertools.Functions.Random.WaitbarConsole as WaitbarConsole
            total_amount = 1
            amount = 0
            WaitbarConsole.printWaitBar(amount, total_amount, prefix = 'Progress:', suffix = 'Complete', length = 50)

        # Download and process the data
        DownloadData(output_folder, latlim, lonlim)

        if Waitbar == 1:
            amount = 1
            WaitbarConsole.printWaitBar(amount, total_amount, prefix = 'Progress:', suffix = 'Complete', length = 50)

    else:
        if Waitbar == 1:
            print("\nSRTM altitude map (3s) already exists in output folder")

if __name__ == '__main__':
    main(sys.argv)



# -*- coding: utf-8 -*-
'''
Authors: Tim Hessels
Module: Products/ETref
'''

# import general python modules
import os
from osgeo import gdal, osr
import numpy as np
import pandas as pd
import subprocess
import netCDF4
import glob

# import watertools modules
from watertools.General import data_conversions as DC
from watertools.General import raster_conversions as RC
from watertools.Products.ETref.SlopeInfluence_ETref import SlopeInfluence

def CollectLANDSAF(SourceLANDSAF, Dir, Startdate, Enddate, latlim, lonlim):
    """
    This function collects and clip LANDSAF data

    Keyword arguments:
    SourceLANDSAF -- 'C:/'  path to the LANDSAF source data (The directory includes SIS and SID)
    Dir -- 'C:/' path to the WA map
    Startdate -- 'yyyy-mm-dd'
    Enddate -- 'yyyy-mm-dd'
    latlim -- [ymin, ymax] (values must be between -60 and 60)
    lonlim -- [xmin, xmax] (values must be between -180 and 180)
    """

    # Make an array of the days of which the ET is taken
    Dates = pd.date_range(Startdate,Enddate,freq = 'D')

    # make directories
    SISdir = os.path.join(Dir,'Landsaf_Clipped','SIS')
    if os.path.exists(SISdir) is False:
        os.makedirs(SISdir)

    SIDdir= os.path.join(Dir,'Landsaf_Clipped','SID')
    if os.path.exists(SIDdir) is False:
        os.makedirs(SIDdir)

    ShortwaveBasin(SourceLANDSAF, Dir, latlim, lonlim, Dates=[Startdate,Enddate])
    DEMmap_str=os.path.join(Dir,'HydroSHED','DEM','DEM_HydroShed_m_3s.tif')
    geo_out, proj, size_X, size_Y = RC.Open_array_info(DEMmap_str)

    # Open DEM map
    demmap = RC.Open_tiff_array(DEMmap_str)
    demmap[demmap<0]=0

    # make lat and lon arrays)
    dlat = geo_out[5]
    dlon = geo_out[1]
    lat = geo_out[3] + (np.arange(size_Y)+0.5)*dlat
    lon = geo_out[0] + (np.arange(size_X)+0.5)*dlon


    for date in Dates:
        # day of year
        day=date.dayofyear
        Horizontal, Sloping, sinb, sinb_hor, fi, slope, ID  = SlopeInfluence(demmap,lat,lon,day)


        SIDname = os.path.join(SIDdir,'SAF_SID_Daily_W-m2_' + date.strftime('%Y-%m-%d') + '.tif')
        SISname = os.path.join(SISdir,'SAF_SIS_Daily_W-m2_' + date.strftime('%Y-%m-%d') + '.tif')

        #PREPARE SID MAPS
        SIDdest = RC.reproject_dataset_example(SIDname,DEMmap_str,method = 3)
        SIDdata=SIDdest.GetRasterBand(1).ReadAsArray()

        #PREPARE SIS MAPS
        SISdest = RC.reproject_dataset_example(SISname,DEMmap_str,method = 3)
        SISdata=SISdest.GetRasterBand(1).ReadAsArray()

        # Calculate ShortWave net
        Short_Wave_Net = SIDdata * (Sloping/Horizontal)+SISdata *86400/1e6

        # Calculate ShortWave Clear
        Short_Wave = Sloping
        Short_Wave_Clear = Short_Wave *(0.75 + demmap * 2 * 10**-5)

        # make directories
        PathClear= os.path.join(Dir,'Landsaf_Clipped','Shortwave_Clear_Sky')
        if os.path.exists(PathClear) is False:
            os.makedirs(PathClear)

        PathNet= os.path.join(Dir,'Landsaf_Clipped','Shortwave_Net')
        if os.path.exists(PathNet) is False:
            os.makedirs(PathNet)

        # name Shortwave Clear and Net
        nameFileNet='ShortWave_Net_Daily_W-m2_' + date.strftime('%Y-%m-%d') + '.tif'
        nameNet= os.path.join(PathNet,nameFileNet)

        nameFileClear='ShortWave_Clear_Daily_W-m2_' + date.strftime('%Y-%m-%d') + '.tif'
        nameClear= os.path.join(PathClear,nameFileClear)

        # Save net and clear short wave radiation
        DC.Save_as_tiff(nameNet, Short_Wave_Net, geo_out, proj)
        DC.Save_as_tiff(nameClear, Short_Wave_Clear, geo_out, proj)
    return


def ShortwaveBasin(SourceLANDSAF, Dir, latlim, lonlim, Dates = ['2000-01-01','2013-12-31']):
    """
    This function creates short wave maps based on the SIS and SID

    Keyword arguments:
    SourceLANDSAF -- 'C:/'  path to the LANDSAF source data (The directory includes SIS and SID)
    Dir -- 'C:/' path to the WA map
    latlim -- [ymin, ymax] (values must be between -60 and 60)
    lonlim -- [xmin, xmax] (values must be between -180 and 180)
    Dates -- ['yyyy-mm-dd','yyyy-mm-dd']
    """

    # Produces shortwave radiation grids for a particular basin or particular bounds
    Types = ['SIS','SID']
    Dates = pd.date_range(Dates[0],Dates[1],freq='D')

    for Type in Types:
        for Date in Dates:

            SAFdir = os.path.join(SourceLANDSAF, Type)
            OutPath =  os.path.join(Dir, 'Landsaf_Clipped', Type, 'SAF_' + Type + '_EuropeAfrica_day_W-m2_' + Date.strftime('%Y-%m-%d') + '.tif')

            if os.path.exists(SAFdir) is False:
                os.mkdir(SAFdir)

            # Convert nc to tiff files
            Transform(SourceLANDSAF, OutPath, Type, Dates = [Date.strftime('%Y-%m-%d'),Date.strftime('%Y-%m-%d')])

            # Get environmental variable
            WA_env_paths = os.environ["WA_PATHS"].split(';')
            GDAL_env_path = WA_env_paths[0]
            GDAL_TRANSLATE_PATH = os.path.join(GDAL_env_path, 'gdal_translate.exe')

            # Define output name
            nameOut= os.path.join(Dir,'Landsaf_Clipped',Type,'SAF_' + Type + '_daily_W-m2_' + Date.strftime('%Y-%m-%d') + '.tif')

            # Create command for cmd
            fullCmd = ' '.join(['"%s" -projwin %s %s %s %s' % (GDAL_TRANSLATE_PATH, lonlim[0]-0.1, latlim[1]+0.1, lonlim[1]+0.1, latlim[0]-0.1), '-of GTiff', OutPath, nameOut])  # -r {nearest}

            # Run command prompt in cmd
            process = subprocess.Popen(fullCmd)
            process.wait()
            print('Landsaf ' + Type + ' file for ' + Date.strftime('%Y-%m-%d') + ' created.')
            os.remove(OutPath)


def Transform(SourceLANDSAF, OutPath, Type, Dates = ['2000-01-01','2013-12-31']):
    """
    This function creates short wave maps based on the SIS and SID
    This function converts packed nc files to gtiff file of comparable file size

    Keyword arguments:
    SourceLANDSAF -- 'C:/'  path to the LANDSAF source data (The directory includes SIS and SID)
    Dir -- 'C:/' path to the WA map
    latlim -- [ymin, ymax] (values must be between -60 and 60)
    lonlim -- [xmin, xmax] (values must be between -180 and 180)
    Dates -- ['yyyy-mm-dd','yyyy-mm-dd']
    """

    path = os.path.join(SourceLANDSAF,Type)

    os.chdir(path)
    Dates = pd.date_range(Dates[0],Dates[1],freq='D')

    srs = osr.SpatialReference()
    srs.SetWellKnownGeogCS("WGS84")
    projection = srs.ExportToWkt()
    driver = gdal.GetDriverByName("GTiff")


    for Date in Dates:
        if Type == 'SIS':
            ZipFile = glob.glob('SISdm%s*.nc.gz' % Date.strftime('%Y%m%d'))[0]
            File = os.path.splitext(ZipFile)[0]
        elif Type == 'SID':
            ZipFile = glob.glob('*dm%s*.nc.gz' % Date.strftime('%Y%m%d'))[0]
            File = os.path.splitext(ZipFile)[0]

        # find path to the executable
        fullCmd = ''.join("7z x %s -o%s -aoa"  %(os.path.join(path,ZipFile),path))
        process = subprocess.Popen(fullCmd)
        process.wait()

        NC = netCDF4.Dataset(File,'r+',format='NETCDF4')
        Data = NC[Type][0,:,:]
        lon = NC.variables['lon'][:][0]	- 0.025
        lat = NC.variables['lat'][:][-1] + 0.025
        geotransform = [lon,0.05,0,lat,0,-0.05]

        dst_ds = driver.Create(OutPath, int(np.size(Data,1)), int(np.size(Data,0)),  1, gdal.GDT_Float32,  ['COMPRESS=DEFLATE'])
        # set the reference info
        dst_ds.SetProjection(projection)
        dst_ds.SetGeoTransform(geotransform)
        dst_ds.GetRasterBand(1).SetNoDataValue(-1)
        dst_ds.GetRasterBand(1).WriteArray(np.flipud(Data))
        NC.close()
        del dst_ds, NC, Data

        os.remove(File)



# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Collect/HiHydroSoil

Restrictions:
The data and this python file may not be distributed to others without
permission of the WA+ team due data restriction of the HiHydroSoil developers.

Description:
This module downloads HiHydroSoil data from
ftp.wateraccounting.unesco-ihe.org. Use the HiHydroSoil function to
download and create monthly HiHydroSoil images in Gtiff format.

Examples:
from watertools.Collect import HiHydroSoil
HiHydroSoil.ThetaSat_TopSoil(Dir='C:/Temp/',  latlim=[-10, 30], lonlim=[-20, -10])
"""

from .ThetaSat_TopSoil import main as ThetaSat_TopSoil

__all__ = ['monthly']

__version__ = '0.1'



# -*- coding: utf-8 -*-
"""
Created on Wed Sep  9 09:31:37 2020

@author: timhe
"""
import os
import sys
import json
import watertools

from cryptography.fernet import Fernet

def GET(server):
    
    path = os.path.dirname(watertools.__file__)
    
    key_file = os.path.join(path, "wa_key.txt")
    
    if not os.path.exists(key_file):
        sys.exit("Run watertools.Set_Up_watertools.create_key()")   
                 
    json_file = os.path.join(path, "keys.json")
    
    if not os.path.exists(json_file):
        sys.exit("Run watertools.Set_Up_watertools.set_up_account()")
        
    f = open(key_file,"r")
    key = f.read()
    f.close()
    
    cipher_suite = Fernet(key.encode('utf-8'))    
    
    # open json file  
    with open(json_file) as f:
        datastore = f.read()
    obj = json.loads(datastore)      
    f.close()
    
    if server != "WAPOR" and server != "NASA_BEARER":
        username_crypt, pwd_crypt = obj[server]
        username = cipher_suite.decrypt(username_crypt.encode("utf-8"))
        pwd = cipher_suite.decrypt(pwd_crypt.encode("utf-8"))        
    else:
        username_crypt = obj[server]
        username = cipher_suite.decrypt(username_crypt[0].encode("utf-8"))
        pwd = b''
    
    return(str(username.decode("utf-8")), str(pwd.decode("utf-8")))


# -*- coding: utf-8 -*-
'''
Authors: Tim Hessels
Module: Products/ETref
'''
# import general python modules
import numpy as np

def SlopeInfluence(DEMmap,latitude,longitude,day):
	
    '''
    This function corrects the solar radiation for the sloping terrain.
    All the formulas are based on Allen (2006)
	
    DEMmap -- path to the DEM map
    latitude -- numpy array with the latitude
    longitude -- numpy array with the longitude
    day -- Day of the year
    '''
	
    # This model calculates the amount of sunlight on a particular day for a sloping
    # terrain. The needed inputs are:
    # DEM > a DEM map
    # lat/lon > either as 2 vectors or 2 matrices
    # day > the day of the year
    # Because we do not know the height of neigbouring cells at the border of the
    # grid, these values can deviate from real values.
    # Be carefull with high latitudes (>66, polar circle)! Calculations for regions without sunset
    # (all day sun) are not calculated correctly.
    
    # Solar constant
    G = 1367.0
    
    # If lat/lon are not a matrix but a vector create matrixes
    if not latitude.shape == longitude.shape:
        latitude = np.tile(latitude.reshape(len(latitude),1),[1,len(longitude)])
        longitude = np.tile(longitude.reshape(1,len(longitude)),[len(latitude),1])
    elif len(latitude.shape) == 1 and len(longitude.shape) == 1:
        latitude = np.tile(latitude.reshape(len(latitude),1),[1,len(longitude)])
        longitude = np.tile(longitude.reshape(1,len(longitude)),[len(longitude),1])
    
    # Convert lat/lon to radians
    lat = np.radians(latitude)
    lon = np.radians(longitude)
           
    # Calculate dlat/dlon assuming a regular grid
    latdiff = np.diff(lat,axis=0) 
    dlat = np.vstack((latdiff[0,:][None,:],(latdiff[1:,:] + latdiff[0:-1,:])/2,latdiff[-1,:][None,:]))
    londiff = np.diff(lon,axis=1) 
    dlon = np.hstack((londiff[:,0][:,None],(londiff[:,1:] + londiff[:,0:-1])/2,londiff[:,-1][:,None]))
    
    # And convert to meters
    lons = np.cos(lat) * np.cos(lat) * np.sin(dlon/2) ** 2
    lats = np.sin(dlat / 2) ** 2
    dlon = 2 * np.arcsin(np.sqrt(lons)) * 6371000
    dlat = 2 * np.arcsin(np.sqrt(lats)) * 6371000
    del lats, lons
    
    # Calculate dy_lat and dy_lon, height differences in latitude and longitude directions.
    latdiff = np.diff(DEMmap,axis=0) 
    dy_lat = np.vstack((latdiff[0,:][None,:],(latdiff[1:,:] + latdiff[0:-1,:])/2,latdiff[-1,:][None,:]))
    londiff = -np.diff(DEMmap,axis=1) 
    dy_lon = np.hstack((londiff[:,0][:,None],(londiff[:,1:] + londiff[:,0:-1])/2,londiff[:,-1][:,None]))    
    
    # Calculate slope
    slope = np.arctan((np.abs(dy_lat) + np.abs(dy_lon)) / np.sqrt(dlon**2+dlat**2))
    
    # declination of earth
    delta = np.arcsin(np.sin(23.45/360*2*np.pi)*np.sin((360.0/365.0)*(day-81)/360*2*np.pi))
    # EQ 2
    D2 = 1 / (1 + 0.033* np.cos(day/365*2*np.pi))
    
    constant =  G / D2 / (2*np.pi) 
    
    # Slope direction
    with np.errstate(divide='ignore'):
        slopedir = np.arctan(dy_lon/dy_lat) 
        
        # Exception if divided by zero    
        slopedir[np.logical_and(dy_lat == 0 , dy_lon <= 0)] = np.pi / 2
        slopedir[np.logical_and(dy_lat == 0, dy_lon > 0)] = -np.pi / 2
        
        # Correction ip dy_lat > 0
        slopedir[np.logical_and(dy_lat > 0, dy_lon < 0)] = np.pi + slopedir[np.logical_and(dy_lat > 0, dy_lon < 0)]
        slopedir[np.logical_and(dy_lat > 0, dy_lon >= 0)] = -np.pi + slopedir[np.logical_and(dy_lat > 0, dy_lon >= 0)]
    
    # Now calculate the expected clear sky radiance day by day for:
    # - A horizontal surface
    # - A mountaneous surface (slopes)
    
    # Define grids
    Horizontal = np.zeros(slope.shape) * np.nan
    Sloping = np.zeros(slope.shape) * np.nan
    
    sinb = np.zeros(slope.shape) *np.nan
    sinb_hor = np.zeros(slope.shape) *np.nan
    
    f1 = np.zeros(slope.shape) *np.nan
    f2 = np.zeros(slope.shape) *np.nan
    f3 = np.zeros(slope.shape) *np.nan
    f4 = np.zeros(slope.shape) *np.nan
    f5 = np.zeros(slope.shape) *np.nan
    
    #######
    ## Horizontal (EQ 35)
    #######
    
    # Check where there is no sunrise and assign values
    Horizontal[np.abs(delta-lat) > np.pi/2] = 0 
    
    # Check whether there are areas without sunset
    ID = np.where(np.ravel(np.abs(delta+lat)) > np.pi/2)    
    sunrise = -np.pi
    sunset = np.pi
    Horizontal.flat[ID] = IntegrateHorizontal((G/(np.pi*D2)),sunrise,sunset,delta,lat.flat[ID])    

    # Calculate sunset and sunrise for all other cells    
    ID = np.where(np.isnan(np.ravel(Horizontal)))    
    bound = BoundsHorizontal(delta,lat.flat[ID])     
    Horizontal.flat[ID] = IntegrateHorizontal((G/(np.pi*D2)),-bound,bound,delta,lat.flat[ID])
    
    ws = np.arccos(-np.tan(delta)*np.tan(lat))
    a,b,c,g,h = Table2(delta,lat,slope,slopedir) 
    sinb_hor = (2*g**2*ws + 4*g*h*np.sin(ws) + h**2*(ws+0.5*np.sin(2*ws))) / (2*(g*ws+h*np.sin(ws)))    
    
    #######
    ## Sloping (EQ )
    #######
    A1,A2,B1,B2 = TwoPeriodSunnyTimes(constant,delta,slope,slopedir,lat)    

    # Check whether we have one or two periods of sunlight.
    TwoPeriod = TwoPeriods(delta,slope,lat) 
    
    ID = np.where(np.ravel(TwoPeriod == True))
    Sloping.flat[ID] = TwoPeriodSun(constant,delta,slope.flat[ID],slopedir.flat[ID],lat.flat[ID])
    f1.flat[ID],f2.flat[ID],f3.flat[ID],f4.flat[ID],f5.flat[ID] = Table1b(A1.flat[ID],A2.flat[ID],B1.flat[ID],B2.flat[ID]) 
    
    ID = np.where(np.ravel(TwoPeriod == False))
    Sloping.flat[ID] = OnePeriodSun(constant,delta,slope.flat[ID],slopedir.flat[ID],lat.flat[ID])
    f1.flat[ID],f2.flat[ID],f3.flat[ID],f4.flat[ID],f5.flat[ID] = Table1a(A1.flat[ID],A2.flat[ID])  

    sinb = (b*g-a*h*f1 - c*g*f2 + (0.5*b*h - a*g)*f3 + 0.25*b*h*f4 + 0.5*c*h*f5) / b*f1 - c*f2 - a*f3
    #sinb[sinb[~np.isnan(sinb)] < 0.0] = 0.0
    
    fi = 0.75 + 0.25*np.cos(slope) - (0.5*slope/np.pi)

    return(Horizontal,Sloping, sinb, sinb_hor, fi, slope, np.where(np.ravel(TwoPeriod == True)))

def Table1a(sunrise,sunset):
    f1 = np.sin(sunset) - np.sin(sunrise)
    f2 = np.cos(sunset) - np.cos(sunrise)
    f3 = sunset - sunrise
    f4 = np.sin(2*sunset) - np.sin(2*sunrise)
    f5 = np.sin(sunset)**2 - np.sin(sunrise)**2
    return f1, f2, f3, f4, f5
    
def Table1b(w1,w2,w1b,w2b):
    f1 = np.sin(w2b) - np.sin(w1) + np.sin(w2) - np.sin(w1b)
    f2 = np.cos(w2b) - np.cos(w1) + np.cos(w2) - np.cos(w1b)
    f3 = w2b - w1 + w2 - w1b
    f4 = np.sin(2*w2b) - np.sin(2*w1) + np.sin(2*w2) - np.sin(2*w1b)
    f5 = np.sin(w2b)**2 - np.sin(w1)**2 + np.sin(w2)**2 - np.sin(w1b)**2
    return f1, f2, f3, f4, f5

def Table2(delta,lat,slope,slopedir):
    a = np.sin(delta)*np.cos(lat)*np.sin(slope)*np.cos(slopedir) - np.sin(delta)*np.sin(lat)*np.cos(slope)
    b = np.cos(delta)*np.cos(lat)*np.cos(slope) + np.cos(delta)*np.sin(lat)*np.sin(slope)*np.cos(slopedir)
    c = np.cos(delta)*np.sin(slopedir)*np.sin(slope)
    g = np.sin(delta)*np.sin(lat)
    h = np.cos(delta)*np.cos(lat)
    
    return a,b,c,g,h

def SunHours(delta,slope,slopedir,lat):
    # Define sun hours in case of one sunlight period
    
    a,b,c = Constants(delta,slope,slopedir,lat)
    riseSlope, setSlope = BoundsSlope(a,b,c)  
    bound = BoundsHorizontal(delta,lat)  

    Calculated = np.zeros(slope.shape, dtype = bool)    
    RiseFinal = np.zeros(slope.shape)    
    SetFinal = np.zeros(slope.shape)
    
    # First check sunrise is not nan
    # This means that their is either no sunrise (whole day night) or no sunset (whole day light)
    # For whole day light, use the horizontal sunrise and whole day night a zero..
    Angle4 = AngleSlope(a,b,c,-bound)    
    RiseFinal[np.logical_and(np.isnan(riseSlope),Angle4 >= 0)] = -bound[np.logical_and(np.isnan(riseSlope),Angle4 >= 0)]
    Calculated[np.isnan(riseSlope)] = True 
    
    # Step 1 > 4    
    Angle1 = AngleSlope(a,b,c,riseSlope)
    Angle2 = AngleSlope(a,b,c,-bound)    
    
    ID = np.ravel_multi_index(np.where(np.logical_and(np.logical_and(Angle2 < Angle1+0.001 ,Angle1 < 0.001),Calculated == False) == True),a.shape)
    RiseFinal.flat[ID] = riseSlope.flat[ID]
    Calculated.flat[ID] = True
    # step 5 > 7
    Angle3 = AngleSlope(a,b,c,-np.pi - riseSlope)
    
    ID = np.ravel_multi_index(np.where(np.logical_and(np.logical_and(-bound<(-np.pi-riseSlope),Angle3 <= 0.001),Calculated == False) == True),a.shape) 
    RiseFinal.flat[ID] = -np.pi -riseSlope.flat[ID]
    Calculated.flat[ID] = True
    
    # For all other values we use the horizontal sunset if it is positive, otherwise keep a zero   
    RiseFinal[Calculated == False] = -bound[Calculated == False] 
    
    # Then check sunset is not nan or < 0 
    Calculated = np.zeros(slope.shape, dtype = bool)     
    
    Angle4 = AngleSlope(a,b,c,bound)    
    SetFinal[np.logical_and(np.isnan(setSlope),Angle4 >= 0)] = bound[np.logical_and(np.isnan(setSlope),Angle4 >= 0)]
    Calculated[np.isnan(setSlope)] = True     
    
    # Step 1 > 4    
    Angle1 = AngleSlope(a,b,c,setSlope)
    Angle2 = AngleSlope(a,b,c,bound)    
    
    ID = np.ravel_multi_index(np.where(np.logical_and(np.logical_and(Angle2 < Angle1+0.001,Angle1 < 0.001),Calculated == False) == True),a.shape)
    SetFinal.flat[ID] = setSlope.flat[ID]
    Calculated.flat[ID] = True
    # step 5 > 7
    Angle3 = AngleSlope(a,b,c,np.pi - setSlope)
    
    ID = np.ravel_multi_index(np.where(np.logical_and(np.logical_and(bound>(np.pi-setSlope),Angle3 <= 0.001),Calculated == False) == True),a.shape) 
    SetFinal.flat[ID] = np.pi - setSlope.flat[ID]
    Calculated.flat[ID] = True
    
    # For all other values we use the horizontal sunset if it is positive, otherwise keep a zero   
    SetFinal[Calculated == False] = bound[Calculated == False]   
    
    #    Angle4 = AngleSlope(a,b,c,bound)    
    #    SetFinal[np.logical_and(Calculated == False,Angle4 >= 0)] = bound[np.logical_and(Calculated == False,Angle4 >= 0)]

    # If Sunrise is after Sunset there is no sunlight during the day
    SetFinal[SetFinal <= RiseFinal] = 0
    RiseFinal[SetFinal <= RiseFinal] = 0
    
    return(RiseFinal,SetFinal)

def OnePeriodSun(constant,delta,slope,slopedir,lat):
    # Initialize function 
    sunrise,sunset = SunHours(delta,slope,slopedir,lat) 
    
    # Finally calculate resulting values
    Vals = IntegrateSlope(constant,sunrise,sunset,delta,slope,slopedir,lat)    
    
    return(Vals)

def TwoPeriodSunnyTimes(constant,delta,slope,slopedir,lat):
    # First derive A1 and A2 from the normal procedure    
    A1,A2 = SunHours(delta,slope,slopedir,lat)
    
    # Then calculate the other two functions.
    # Initialize function    
    
    a,b,c = Constants(delta,slope,slopedir,lat)
    riseSlope, setSlope = BoundsSlope(a,b,c)  
    
    B1 = np.maximum(riseSlope,setSlope)
    B2 = np.minimum(riseSlope,setSlope)
    
    Angle_B1 = AngleSlope(a,b,c,B1)
    Angle_B2 = AngleSlope(a,b,c,B2) 
    
    B1[abs(Angle_B1) > 0.001] = np.pi - B1[abs(Angle_B1) > 0.001]
    B2[abs(Angle_B2) > 0.001] = -np.pi - B2[abs(Angle_B2) > 0.001]    
    
    # Check if two periods really exist
    ID = np.ravel_multi_index(np.where(np.logical_and(B2 >= A1, B1 <= A2) == True),a.shape)
    Val = IntegrateSlope(constant,B2.flat[ID],B1.flat[ID],delta,slope.flat[ID],slopedir.flat[ID],lat.flat[ID])
    ID = ID[Val < 0]
    
    return A1,A2,B1,B2

def TwoPeriodSun(constant,delta,slope,slopedir,lat):
    # First derive A1 and A2 from the normal procedure    
    A1,A2 = SunHours(delta,slope,slopedir,lat)
    
    # Then calculate the other two functions.
    # Initialize function    
    
    a,b,c = Constants(delta,slope,slopedir,lat)
    riseSlope, setSlope = BoundsSlope(a,b,c)  
    
    B1 = np.maximum(riseSlope,setSlope)
    B2 = np.minimum(riseSlope,setSlope)
    
    Angle_B1 = AngleSlope(a,b,c,B1)
    Angle_B2 = AngleSlope(a,b,c,B2) 
    
    B1[abs(Angle_B1) > 0.001] = np.pi - B1[abs(Angle_B1) > 0.001]
    B2[abs(Angle_B2) > 0.001] = -np.pi - B2[abs(Angle_B2) > 0.001]    
    
    # Check if two periods really exist
    ID = np.ravel_multi_index(np.where(np.logical_and(B2 >= A1, B1 >= A2) == True),a.shape)
    Val = IntegrateSlope(constant,B2.flat[ID],B1.flat[ID],delta,slope.flat[ID],slopedir.flat[ID],lat.flat[ID])
    ID = ID[Val < 0]
    
    # Finally calculate resulting values
    Vals = np.zeros(B1.shape)

    Vals.flat[ID] = (IntegrateSlope(constant,A1.flat[ID],B2.flat[ID],delta,slope.flat[ID],slopedir.flat[ID],lat.flat[ID])  + 
                       IntegrateSlope(constant,B1.flat[ID],A2.flat[ID],delta,slope.flat[ID],slopedir.flat[ID],lat.flat[ID]))
    ID = np.ravel_multi_index(np.where(Vals == 0),a.shape)   
    Vals.flat[ID] = IntegrateSlope(constant,A1.flat[ID],A2.flat[ID],delta,slope.flat[ID],slopedir.flat[ID],lat.flat[ID])
    
    return(Vals)

def Constants(delta,slope,slopedir,lat):
    # Equation 11
    a = np.sin(delta)*np.cos(lat)*np.sin(slope)*np.cos(slopedir) - np.sin(delta)*np.sin(lat)*np.cos(slope)
    b = np.cos(delta)*np.cos(lat)*np.cos(slope) + np.cos(delta)*np.sin(lat)*np.sin(slope)*np.cos(slopedir)
    c = np.cos(delta)*np.sin(slope)*np.sin(slopedir)
    return(a,b,c)

def AngleSlope(a,b,c,time):
    angle = -a + b*np.cos(time) + c*np.sin(time)
    return(angle)
    
def AngleHorizontal(delta,lat,time):
    angle = np.sin(delta)*np.sin(lat)+np.cos(delta)*np.cos(lat)*np.cos(time)
    return(angle)

def BoundsSlope(a,b,c):
    #Equation 13
    Div = (b**2+c**2)   
    Div[Div <= 0] = 0.00001
    sinB = (a*c + b*np.sqrt(b**2+c**2-a**2)) / Div
    sinA = (a*c - b*np.sqrt(b**2+c**2-a**2)) / Div
    
    sinB[sinB < -1] = -1; sinB[sinB > 1] = 1
    sinA[sinA < -1] = -1; sinA[sinA > 1] = 1
    
    sunrise = np.arcsin(sinA)
    sunset = np.arcsin(sinB)
    return(sunrise,sunset)
    
def TwoPeriods(delta,slope,lat):
    # Equation 7
    TwoPeriod = (np.sin(slope) > np.ones(slope.shape)*np.sin(lat)*np.cos(delta)+np.cos(lat)*np.sin(delta))
    return(TwoPeriod)
    
def BoundsHorizontal(delta,lat):
    # This function calculates sunrise hours based on earth inclination and latitude
    # If there is no sunset or sunrise hours the values are either set to 0 (polar night)
    # or pi (polar day)    
    bound = np.arccos(-np.tan(delta)*np.tan(lat))
    bound[abs(delta+lat) > np.pi/2] = np.pi
    bound[abs(delta-lat) > np.pi/2] = 0
    
    return(bound)

def IntegrateHorizontal(constant,sunrise,sunset,delta,lat):
    # Equation 4 & 6
    ws = np.arccos(-np.tan(delta)*np.tan(lat))
    integral = constant * (np.sin(delta)*np.sin(lat)*ws + np.cos(delta)*np.cos(lat)*np.sin(ws))
#    integral = constant * (np.sin(delta)*np.sin(lat)*(sunset-sunrise) 
#                + np.cos(delta)*np.cos(lat)*(np.sin(sunset)-np.sin(sunrise)))
    return(integral)

def IntegrateNormal(constant,sunrise,sunset):
    integral = constant * (sunset - sunrise)
    return(integral)

def IntegrateSlope(constant,sunrise,sunset,delta,slope,slopedir,lat):
    # Equation 5 & 6
    integral = constant * (np.sin(delta)*np.sin(lat)*np.cos(slope)*(sunset-sunrise) 
                - np.sin(delta)*np.cos(lat)*np.sin(slope)*np.cos(slopedir)*(sunset-sunrise)
                + np.cos(delta)*np.cos(lat)*np.cos(slope)*(np.sin(sunset)-np.sin(sunrise))
                + np.cos(delta)*np.sin(lat)*np.sin(slope)*np.cos(slopedir)*(np.sin(sunset)-np.sin(sunrise))
                - np.cos(delta)*np.sin(slope)*np.sin(slopedir)*(np.cos(sunset)-np.cos(sunrise)))
    return(integral)


"""
Authors: Tim Hessels
Module: Collect/JRC
"""

import sys
from watertools.Collect.JRC.DataAccess import DownloadData


def main(Dir, latlim, lonlim, Waitbar = 1):
    """
    This function downloads JRC water occurrence data for the specified spatial extent.

    Keyword arguments:
    Dir -- 'C:/file/to/path/'
    latlim -- [ymin, ymax]
    lonlim -- [xmin, xmax]
    Waitbar -- 1 (Default) will print a waitbar
    """
    print('\nDownload JRC occurrence map')
    DownloadData(Dir, latlim, lonlim, Waitbar)

if __name__ == '__main__':
    main(sys.argv)


# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Collect/RFE
"""

import numpy as np
import os
import pandas as pd
from ftplib import FTP
from joblib import Parallel, delayed

import watertools.General.data_conversions as DC
import watertools.General.raster_conversions as RC

def DownloadData(Dir, Startdate, Enddate, latlim, lonlim, Waitbar, cores):
    """
    This function downloads RFE daily or monthly data

    Keyword arguments:
    Dir -- 'C:/file/to/path/'
    Startdate -- 'yyyy-mm-dd'
    Enddate -- 'yyyy-mm-dd'
    latlim -- [ymin, ymax] (values must be between -50 and 50)
    lonlim -- [xmin, xmax] (values must be between -180 and 180)
    cores -- The number of cores used to run the routine. It can be 'False'
             to avoid using parallel computing routines.
    TimeCase -- String equal to 'daily' or 'monthly'
    Waitbar -- 1 (Default) will print a waitbar
    """

	# Check variables
    if not Startdate:
        Startdate = pd.Timestamp('2001-01-01')
    if not Enddate:
        Enddate = pd.Timestamp('Now')
    Dates = pd.date_range(Startdate,  Enddate, freq='D')

    # Create Waitbar
    if Waitbar == 1:
        import watertools.Functions.Random.WaitbarConsole as WaitbarConsole
        total_amount = len(Dates)
        amount = 0
        WaitbarConsole.printWaitBar(amount, total_amount, prefix = 'Progress:', suffix = 'Complete', length = 50)

    if latlim[0] < -40.05 or latlim[1] > 40.05:
        print('Latitude above 50N or below 50S is not possible.'
               ' Value set to maximum')
        latlim[0] = np.max(latlim[0], -40.05)
        latlim[1] = np.min(lonlim[1], 40.05)
    if lonlim[0] < -20.05 or lonlim[1] > 55.05:
        print('Longitude must be between 180E and 180W.'
               ' Now value is set to maximum')
        lonlim[0] = np.max(latlim[0], -20.05)
        lonlim[1] = np.min(lonlim[1], 55.05)

	 # Make directory
    output_folder = os.path.join(Dir, 'Precipitation', 'RFE', 'Daily/')
    if not os.path.exists(output_folder):
        os.makedirs(output_folder)


    # Define IDs
    yID = 801 - np.int16(np.array([np.ceil((latlim[1] + 40.05)*10),
                                    np.floor((latlim[0] + 40.05)*10)-1]))
    xID = np.int16(np.array([np.floor((lonlim[0] + 20.05)*10),
                             np.ceil((lonlim[1] + 20.05)*10)+1]))

    # Pass variables to parallel function and run
    args = [output_folder, lonlim, latlim, xID, yID]

    if not cores:
        for Date in Dates:
            RetrieveData(Date, args)
            if Waitbar == 1:
                amount += 1
                WaitbarConsole.printWaitBar(amount, total_amount, prefix = 'Progress:', suffix = 'Complete', length = 50)
        results = True
    else:
        results = Parallel(n_jobs=cores)(delayed(RetrieveData)(Date, args)
                                         for Date in Dates)

    return results


def RetrieveData(Date, args):
    """
    This function retrieves RFE data for a given date from the
    ftp://disc2.nascom.nasa.gov server.

    Keyword arguments:
    Date -- 'yyyy-mm-dd'
    args -- A list of parameters defined in the DownloadData function.
    """
    # Argument
    [output_folder, lonlim, latlim, xID, yID] = args

    # Create https
    DirFile = os.path.join(output_folder,'P_RFE.v2.0_mm-day-1_daily_%s.%02s.%02s.tif' %(Date.strftime('%Y'), Date.strftime('%m'), Date.strftime('%d')))

    if not os.path.isfile(DirFile):
        # open ftp server
        ftp = FTP("ftp.cpc.ncep.noaa.gov", "", "")
        ftp.login()

    	 # Define FTP path to directory
        pathFTP = 'fews/fewsdata/africa/rfe2/geotiff/'

        # find the document name in this directory
        ftp.cwd(pathFTP)
        listing = []

        # read all the file names in the directory
        ftp.retrlines("LIST", listing.append)

    	  # create all the input name (filename) and output (outfilename, filetif, DiFileEnd) names
        filename = 'africa_rfe.%s%02s%02s.tif.zip' %(Date.strftime('%Y'), Date.strftime('%m'), Date.strftime('%d'))
        outfilename = os.path.join(output_folder,'africa_rfe.%s%02s%02s.tif' %(Date.strftime('%Y'), Date.strftime('%m'), Date.strftime('%d')))

        try:
            local_filename = os.path.join(output_folder, filename)
            lf = open(local_filename, "wb")
            ftp.retrbinary("RETR " + filename, lf.write)
            lf.close()

            # unzip the file
            zip_filename = os.path.join(output_folder, filename)
            DC.Extract_Data(zip_filename, output_folder)

            # open tiff file
            dataset = RC.Open_tiff_array(outfilename)

            # clip dataset to the given extent
            data = dataset[yID[0]:yID[1], xID[0]:xID[1]]
            data[data < 0] = -9999

            # save dataset as geotiff file
            latlim_adj = 40.05 - 0.1 * yID[0]
            lonlim_adj = -20.05 + 0.1 * xID[0]
            geo = [lonlim_adj, 0.1, 0, latlim_adj, 0, -0.1]
            DC.Save_as_tiff(name=DirFile, data=data, geo=geo, projection="WGS84")

            # delete old tif file
            os.remove(outfilename)
            os.remove(zip_filename)

        except:
            print("file not exists")


    return True



# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Collect/ECMWF

Before running this tool, please install the ecmwfapi. You can find the manual here:
https://software.ecmwf.int/wiki/display/WEBAPI/Access+ECMWF+Public+Datasets

from watertools.Collect import ECMWF
ECMWF.monthly(Dir="C:/test2", Vars = ['T'], Startdate = '2008-01-01' , Enddate = '2008-02-02', latlim = [30.1, 50.1], lonlim = [-20.1, 20.1])
ECMWF.daily(Dir="C:/test2", Vars = ['10U','10V','2D','SR','AL','HCC','TP'], Startdate = '2003-01-01' , Enddate = '2003-02-28', latlim = [30.1, 50.1], lonlim = [20, 40])

Suported parameters are: 'T','SP','Q','SSR','R','E','SUND','RO','TP', '10U', '10V, '2D', 'SR', 'AL', 'HCC' More will be follow soon.
The list of parameters can be found here: https://rda.ucar.edu/cgi-bin/transform?xml=/metadata/ParameterTables/WMO_GRIB1.98-0.128.xml&view=gribdoc
"""

from .daily import main as daily
from .monthly import main as monthly
from .DataAccess import VariablesInfo as VarInfo

__all__ = ['daily', 'monthly', 'VarInfo']

__version__ = '0.1'



"""
Description:
This tool collects FAO's WAPOR data of LEVEL 1 and LEVEL 2 (excluding the seasonal paramters). The output will be tiff files and no conversions is 
preformed to the units. The output are in the same units as on the WAPOR Portal (https://wapor.apps.fao.org/home/1)

The following input is required:
    
    -- output_folder: (string) to define the folder where to save the created tiff files.
    -- Startdate: (string) defines the startdate of the required dataset in the following format "yyyy-mm-dd"
    -- Enddate: (string) defines the enddate of the required dataset in the following format "yyyy-mm-dd"
    -- Latlim: (array) defines the latitude limits in the following format [Latitude_minimum, Latitude_maximum] e.g. [10, 13]
    -- Lonlim: (array) defines the longitude limits in the following format [Longitude_minimum, Longitude_maximum] e.g. [10, 13]
    -- API_KEY: (string) this is a private API KEY that can be collected here: https://io.apps.fao.org/gismgr/api/v1/swagger-ui.html#/IAM/apiKeySignIn
    -- Parameter: (string) define the parameter that is required
    -- version: (string) default is 2, but if version 1 is required use this parameter (options are "1" or "2")
    
To get a describtion of all the possible parameters type:
    
import WaporAPI
WaporAPI.Collect.VariablesInfo.descriptions 

Examples:
import watertools
watertools.Collect.WAPOR.Get_Layer(r"G:\Project_MetaMeta\Input_Data2", "2011-01-01", "2011-02-01", [8.2, 8.7], [39, 39.5], "L2_T_D")
"""

from .Get_Layer import main as Get_Layer


__all__ = ['Get_Layer']
__version__ = '0.0.0'